from datetime import datetime
import json

from lettuce import world
from lettuce.terrain import after, before


def enable(filename=None):
    filename = filename or "lettucetests.json"

    @before.all
    def before_all():
        """
        Set `world._started` to `datetime.now()` to track total duration.
        """
        world._started = datetime.now()

    @after.all
    def generate_json_output(total):
        """
        This callback is called after all the features are
        ran.
        """
        world._stopped = datetime.now()
        total_dict = total_result_to_dict(total)
        with open(filename, "w") as handle:
            json.dump(total_dict, handle)

    @before.each_feature
    @before.each_scenario
    @before.each_step
    def before_each_element(*args):
        """
        Set `step._started`, `scenario._started` or `feature._started` to `datetime.now()`
        to track step/scenario/feature duration.
        """
        element = args[0]
        element._started = datetime.now()

    @after.each_feature
    @after.each_scenario
    @after.each_step
    def after_each_element(*args):
        """
        Set `step._stopped`, `scenario._stopped` or `feature._stopped` to `datetime.now()`
        to track step/scenario/feature duration.
        """
        element = args[0]
        element._stopped = datetime.now()


def total_result_to_dict(total):
    """
    Transform a `TotalResult` to a json-serializable Python dictionary.

    :param total:               a `TotalResult` instance
    :return:                    a Python dictionary
    """
    return {
        "meta": extract_meta(total),
        "duration": _get_duration(world),
        "features": [
            extract_feature_data(feature_result)
            for feature_result in total.feature_results
        ]
    }


def extract_feature_data(feature_result):
    """
    Extract data from a `FeatureResult` instance.

    :param feature_result:                a `FeatureResult` instance
    :return:                              a Python dictionary
    """
    scenarios = []
    meta = {
        "steps": {
            "total": 0,
            "success": 0,
            "failures": 0,
            "skipped": 0,
            "undefined": 0,
        },
        "scenarios": {
            "total": 0,
            "success": 0,
            "failures": 0,
            "skipped": 0,
            "undefined": 0,
        }
    }
    for scenario_result in feature_result.scenario_results:
        scenario_data = extract_scenario_data(scenario_result)
        scenarios.append(scenario_data)
        # scenarios
        success = (
            not scenario_data["meta"]["failures"] and
            not scenario_data["meta"]["skipped"] and
            not scenario_data["meta"]["undefined"]
        )
        meta["scenarios"]["total"] += 1 if scenario_data["meta"]["total"] else 0
        meta["scenarios"]["success"] += 1 if success else 0
        meta["scenarios"]["failures"] += 1 if scenario_data["meta"]["failures"] else 0
        meta["scenarios"]["skipped"] += 1 if scenario_data["meta"]["skipped"] else 0
        meta["scenarios"]["undefined"] += 1 if scenario_data["meta"]["undefined"] else 0
        # steps
        meta["steps"]["total"] += scenario_data["meta"]["total"]
        meta["steps"]["success"] += scenario_data["meta"]["success"]
        meta["steps"]["failures"] += scenario_data["meta"]["failures"]
        meta["steps"]["skipped"] += scenario_data["meta"]["skipped"]
        meta["steps"]["undefined"] += scenario_data["meta"]["undefined"]

    return {
        "name": feature_result.feature.name,
        "duration": _get_duration(feature_result.feature),
        "meta": meta,
        "scenarios": scenarios,
        "background": extract_background_data(feature_result.feature.background)
    }

def extract_background_data(background):
    """
    Extract data from a `Background` instance.

    :param background:                   a `Background` instance, possibly None
    :return:                             a Python dictionary
    """
    if not background:
        return None

    step_data = [extract_step_data(step) for step in background.steps]
    return {
        "meta": {
            "total": len(background.steps),
            "success": sum([s["meta"]["success"] for s in step_data]),
            "failures": sum([s["meta"]["failed"] for s in step_data]),
            "skipped": sum([s["meta"]["skipped"] for s in step_data]),
            "undefined": sum([s["meta"]["undefined"] for s in step_data]),
        },
        "steps": step_data
    }


def extract_scenario_data(scenario_result):
    """
    Extract data from a `ScenarioResult` instance.

    :param scenario_result:              a `ScenarioResult` instance
    :return:                             a Python dictionary
    """
    return {
        "name": scenario_result.scenario.name,
        "duration": _get_duration(scenario_result.scenario),
        "outline": scenario_result.outline,
        "meta": {
            "total": scenario_result.total_steps,
            "success": len(scenario_result.steps_passed),
            "failures": len(scenario_result.steps_failed),
            "skipped": len(scenario_result.steps_skipped),
            "undefined": len(scenario_result.steps_undefined),
        },
        "steps": [extract_step_data(step) for step in scenario_result.all_steps]
    }


def extract_step_data(step):
    """
    Extract data from a `Step` instance.

    :param step:                         a `Step` instance
    :return                              a Python dictionary
    """
    step_data = {
        "name": step.sentence,
        "duration": _get_duration(step),
        "meta": {
            "success": bool(step.passed),
            "failed": bool(step.failed),
            "skipped": not step.passed and not step.failed and step.has_definition,
            "undefined": not step.has_definition,
        },
        "failure": {}
    }
    if step.why:
        step_data["failure"] = {
            "exception": repr(step.why.exception),
            "traceback": step.why.traceback
        }
    return step_data


def extract_meta(total):
    """
    Extract metadata from the `TotalResult`.

    :param total:               a `TotalResult` instance
    :return:                    a Python dictionary
    """
    return {
        "features": {
            "total": total.features_ran,
            "success": total.features_passed,
            "failures": total.features_ran - total.features_passed,
        },
        "scenarios": {
            "total": total.scenarios_ran,
            "success": total.scenarios_passed,
            "failures": total.scenarios_ran - total.scenarios_passed,
        },
        "steps": {
            "total": total.steps,
            "success": total.steps_passed,
            "failures": total.steps_failed,
            "skipped": total.steps_skipped,
            "undefined": total.steps_undefined,
        },
        "is_success": total.is_success,
    }


def _get_duration(element):
    """
    Return the duration of an element.

    :param element:          either a step or a scenario or a feature
    """
    return (element._stopped - element._started).seconds if hasattr(element, '_started') else None



import asyncore
import threading

from email.header import decode_header
from email import message_from_string

from smtpd import SMTPServer


from lettuce import after, before


def _parse_header(val):
    result = ''
    elements = decode_header(val)
    for el, enc in elements:
        result += el.decode(enc) if enc and el else el
    return result


def _decode_part(part):
    content = part.get_payload(decode=True)
    charset = part.get_content_charset()
    return content.decode(charset) if charset else content


def _get_content(msg):
    alternatives = []
    if msg.get_content_type() == 'multipart/alternative':
        payload = msg.get_payload()
        body = _decode_part(payload.pop(0))
        alternatives = [_decode_part(part) for part in payload]
    else:
        body = _decode_part(msg)
    return body, alternatives


def _convert_to_django_msg(msg):

    from django.core.mail import EmailMessage, EmailMultiAlternatives

    body, alternatives = _get_content(msg)
    if alternatives:
        email = EmailMultiAlternatives(body=body,
                                       alternatives=alternatives)
    else:
        email = EmailMessage(body=body)
    email.subject = _parse_header(msg['Subject'])
    email.to = _parse_header(msg['To'])
    email.cc = _parse_header(msg.get('Cc', None))
    email.bcc = _parse_header(msg.get('Bcc', None))
    email.from_email = _parse_header(msg['From'])
    return email


def enable():

    from django.conf import settings
    from lettuce.django import mail

    smtp_queue_server = None

    @before.each_scenario
    def start_server(*args, **kwargs):
        global smtp_queue_server
        smtp_queue_server = QueueSMTPServer((settings.LETTUCE_SMTP_QUEUE_HOST,
                                             settings.LETTUCE_SMTP_QUEUE_PORT),
                                            None)
        smtp_queue_server.start()

    @after.each_scenario
    def stop_server(*args, **kwargs):
        global smtp_queue_server
        smtp_queue_server.stop()

    class QueueSMTPServer(SMTPServer, threading.Thread):
        """
        Asyncore SMTP server wrapped into a thread.
        It pushes incoming email messages into lettuce email queue.
        Based on DummyFTPServer from:
        http://svn.python.org/view/python/branches/py3k/Lib/test/test_ftplib.py?revision=86061&view=markup
        """
        def __init__(self, *args, **kwargs):
            threading.Thread.__init__(self)
            SMTPServer.__init__(self, *args, **kwargs)
            self.active_lock = threading.Lock()
            self.active = False
            self.daemon = True

        def process_message(self, peer, mailfrom, rcpttos, data):
            msg = message_from_string(data)
            django_msg = _convert_to_django_msg(msg)
            mail.queue.put(django_msg)

        def start(self):
            assert not self.active
            self.__flag = threading.Event()
            threading.Thread.start(self)

        def run(self):
            self.active = True
            self.__flag.set()
            while self.active and asyncore.socket_map:
                self.active_lock.acquire()
                asyncore.loop(timeout=0.1, count=1)
                self.active_lock.release()
            asyncore.close_all()

        def stop(self):
            assert self.active
            self.active = False
            self.join()



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import re
import sys


from lettuce import core
from lettuce import strings
from lettuce import terminal
from lettuce import build_failed_first_case

from lettuce.terrain import after
from lettuce.terrain import before
from lettuce.terrain import world

import six


def wrt(what):
    if six.PY2:
        if isinstance(what, unicode):
            what = what.encode('utf-8')
    if six.PY3:
        if isinstance(what, bytes):
            print(what)
            what = what.decode('utf-8')
    world.case.append(what)
    sys.stdout.write(what)


def wrap_file_and_line(string, start, end):
    return re.sub(r'([#] [^:]+[:]\d+)', '%s\g<1>%s' % (start, end), string)


def wp(l):
    if l.startswith("\033[1;32m"):
        l = l.replace(" |", "\033[1;37m |\033[1;32m")
    if l.startswith("\033[1;36m"):
        world.skip_flag = True
        l = l.replace(" |", "\033[1;37m |\033[1;36m")
    if l.startswith("\033[0;36m"):
        world.skip_flag = True
        l = l.replace(" |", "\033[1;37m |\033[0;36m")
    if l.startswith("\033[0;31m"):
        world.fail_flag = True
        l = l.replace(" |", "\033[1;37m |\033[0;31m")
    if l.startswith("\033[1;30m"):
        l = l.replace(" |", "\033[1;37m |\033[1;30m")
    if l.startswith("\033[1;31m"):
        world.fail_flag = True
        l = l.replace(" |", "\033[1;37m |\033[0;31m")  
  
    return l


def write_out(what):

    wrt(wp(what))


@before.each_step
def print_step_running(step):
    if not step.defined_at or not step.display:
        return

    color = '\033[1;30m'

    if step.scenario and step.scenario.outlines:
        color = '\033[0;36m'

    string = step.represent_string(step.original_sentence)
    string = wrap_file_and_line(string, '\033[1;30m', '\033[0m')
    write_out("%s%s" % (color, string))
    if step.hashes and step.defined_at:
        for line in step.represent_hashes().splitlines():
            write_out("\033[1;30m%s\033[0m\n" % line)
            # world.case.append(line)


@after.each_step
def print_step_ran(step):
    if not step.display:
        return
    if step.scenario and step.scenario.outlines and (step.failed or step.passed or step.defined_at):
        return

    if step.hashes and step.defined_at:
        write_out("\033[A" * (len(step.hashes) + 1))

    string = step.represent_string(step.original_sentence)

    if not step.failed:
        string = wrap_file_and_line(string, '\033[1;30m', '\033[0m')

    prefix = '\033[A'
    width, height = terminal.get_size()
    lines_up = len(string) / float(width)
    if lines_up < 1:
        lines_up = 1
    else:
        lines_up = int(lines_up) + 1

    #prefix = prefix * lines_up

    if step.failed:
        color = "\033[0;31m"
        string = wrap_file_and_line(string, '\033[1;41;33m', '\033[0m')
        world.fail_flag = True

    elif step.passed:
        color = "\033[1;32m"

    elif step.defined_at:
        color = "\033[0;36m"
        world.skip_flag = True

    else:
        color = "\033[0;33m"
        prefix = ""

    write_out("%s%s%s" % (prefix, color, string))

    if step.hashes:
        for line in step.represent_hashes().splitlines():
            write_out("%s%s\033[0m\n" % (color, line))

    if step.failed:
        wrt("\033[1;31m")
        pspaced = lambda x: wrt("%s%s" % (" " * step.indentation, x))
        lines = step.why.traceback.splitlines()

        for pindex, line in enumerate(lines):
            pspaced(line)
            if pindex + 1 < len(lines):
                wrt("\n")

        wrt("\033[0m\n")


@before.each_scenario
def print_scenario_running(scenario):
    if scenario.background:
        # Only print the background on the first scenario run
        # So, we determine if this was called previously with the attached background.
        # If so, skip the print_scenario() since we'll call it again in the after_background.
        if not hasattr(world, 'background_scenario_holder'):
            world.background_scenario_holder = {}
        if scenario.background not in world.background_scenario_holder:
            # We haven't seen this background before, add our 1st scenario
            world.background_scenario_holder[scenario.background] = scenario
            return
    string = scenario.represented()
    string = wrap_file_and_line(string, '\033[1;30m', '\033[0m')
    write_out("\n\033[1;37m%s" % string)


@after.outline
def print_outline(scenario, order, outline, reasons_to_fail):
    table = strings.dicts_to_string(scenario.outlines, scenario.keys)
    lines = table.splitlines()
    head = lines.pop(0)

    wline = lambda x: write_out("\033[0;36m%s%s\033[0m\n" % (" " * scenario.table_indentation, x))
    wline_success = lambda x: write_out("\033[1;32m%s%s\033[0m\n" % (" " * scenario.table_indentation, x))
    wline_red_outline = lambda x: write_out("\033[1;31m%s%s\033[0m\n" % (" " * scenario.table_indentation, x))
    wline_red = lambda x: write_out("%s%s" % (" " * scenario.table_indentation, x))
    if order == 0:
        wrt("\n")
        wrt("\033[1;37m%s%s:\033[0m\n" % (" " * scenario.indentation, scenario.language.first_of_examples))
        wline(head)

    line = lines[order]
    if reasons_to_fail:
        wline_red_outline(line)
    else:
        wline_success(line)
    if reasons_to_fail:
        elines = reasons_to_fail[0].traceback.splitlines()
        wrt("\033[1;31m")
        for pindex, line in enumerate(elines):
            wline_red(line)
            if pindex + 1 < len(elines):
                wrt("\n")

        wrt("\033[0m\n")


@before.each_feature
def print_feature_running(feature):
    string = feature.represented()
    lines = string.splitlines()

    write_out("\n")
    for line in lines:
        line = wrap_file_and_line(line, '\033[1;30m', '\033[0m')
        write_out("\033[1;37m%s\n" % line)

@after.harvest
@after.all
def print_end(total=None):
    if total is None:
        return
    write_out("\n")
    if isinstance(total, core.SummaryTotalResults):
        word = total.features_ran_overall > 1 and "features" or "feature"

        color = "\033[1;32m"
        if total.features_passed_overall == 0:
            color = "\033[0;31m"

        write_out("\033[1;37mTest Suite Summary:\n")
        write_out("\033[1;37m%d %s (%s%d passed\033[1;37m)\033[0m\n" % (
            total.features_ran_overall,
            word,
            color,
            total.features_passed_overall))

    else:
        word = total.features_ran > 1 and "features" or "feature"

        color = "\033[1;32m"
        if total.features_passed == 0:
            color = "\033[0;31m"

        write_out("\033[1;37m%d %s (%s%d passed\033[1;37m)\033[0m\n" % (
            total.features_ran,
            word,
            color,
            total.features_passed))

    color = "\033[1;32m"
    if total.scenarios_passed == 0:
        color = "\033[0;31m"

    word = total.scenarios_ran > 1 and "scenarios" or "scenario"
    write_out("\033[1;37m%d %s (%s%d passed\033[1;37m)\033[0m\n" % (
        total.scenarios_ran,
        word,
        color,
        total.scenarios_passed))

    steps_details = []
    kinds_and_colors = (
        ('failed', '\033[0;31m'),
        ('skipped', '\033[0;36m'),
        ('undefined', '\033[0;33m'),
    )

    for kind, color in kinds_and_colors:
        attr = 'steps_%s' % kind
        stotal = getattr(total, attr)
        if stotal:
            steps_details.append("%s%d %s" % (color, stotal, kind))

    steps_details.append("\033[1;32m%d passed\033[1;37m" % total.steps_passed)
    word = total.steps > 1 and "steps" or "step"
    content = "\033[1;37m, ".join(steps_details)

    word = total.steps > 1 and "steps" or "step"
    write_out("\033[1;37m%d %s (%s)\033[0m\n" % (
        total.steps,
        word,
        content))

    if total.proposed_definitions:
        wrt("\n\033[0;33mYou can implement step definitions for undefined steps with these snippets:\n\n")
        wrt("# -*- coding: utf-8 -*-\n")
        wrt("from lettuce import step\n\n")

        last = len(total.proposed_definitions) - 1
        for current, step in enumerate(total.proposed_definitions):
            method_name = step.proposed_method_name
            wrt("@step(u'%s')\n" % step.proposed_sentence)
            wrt("def %s:\n" % method_name)
            wrt("    assert False, 'This step must be implemented'")
            if current is last:
                wrt("\033[0m")

            wrt("\n")

    if total.failed_scenario_locations:
        # print list of failed scenarios, with their file and line number
        wrt("\n")
        wrt("\033[1;31m")
        wrt("List of failed scenarios:\n")
        wrt("\033[0;31m")
        for scenario in total.failed_scenario_locations:
            wrt(scenario)
        wrt("\033[0m")
        wrt("\n")
    # build_failed_first_case()
    try:
        build_failed_first_case()
    except Exception as e:
        print(f'colored_shell_output.py ::build_failed_first_case()出错了，原因：{e}')


def print_no_features_found(where):
    where = core.fs.relpath(where)
    if not where.startswith(os.sep):
        where = '.%s%s' % (os.sep, where)

    write_out('\033[1;31mOops!\033[0m\n')
    write_out(
        '\033[1;37mcould not find features at '
        '\033[1;33m%s\033[0m\n' % where)


@before.each_background
def print_background_running(background):
    wrt('\n')
    wrt('\033[1;37m')
    wrt(background.represented())
    wrt('\033[0m\n')


@after.each_background
def print_first_scenario_running(background, results):
    scenario = world.background_scenario_holder[background]
    print_scenario_running(scenario)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
from lettuce.terrain import after


def enable(runner):
    @after.each_step
    def failfast_or_pdb(step):
        has_traceback = step.why

        if not has_traceback:
            return

        sys.stdout.write(step.why.traceback + '\n')

        try:
            from IPython.core.debugger import Pdb
            pdb = Pdb()
        except ImportError:
            try:
                from IPython.Debugger import Pdb
                from IPython.Shell import IPShell
                IPShell(argv=[''])
                pdb = Pdb()
            except ImportError:
                import pdb

        matched, defined = step.pre_run(False)
        if matched:
            args = matched.groups()
            kwargs = matched.groupdict()
            pdb.runcall(defined.function, step, *args, **kwargs)



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import datetime
import time
import os
import re
import threading
import traceback
import shutil

from lettuce.exceptions import StepLoadingError
import six
world = threading.local()
world._set = False


def _function_matches(one, other):

    if hasattr(one, 'func_code'):
        one_func_code = one.func_code
    else:
        one_func_code = six.get_function_code(one)

    if hasattr(other, 'func_code'):
        other_func_code = other.func_code
    else:
        other_func_code = six.get_function_code(other)

    return (os.path.abspath(one_func_code.co_filename) == os.path.abspath(other_func_code.co_filename) and
            one_func_code.co_firstlineno == other_func_code.co_firstlineno)


def build_case_old(scenario):

    with open(f'{world.cache_path}/res.txt', 'a+', encoding='utf8') as f:
        with open(f'{world.cache_path}/error_case.feature', 'a+', encoding='utf8') as c:
            with open(f'{world.cache_path}/temp_case.feature', 'a+', encoding='utf8') as t:
                _case = world.case[:]
                temp_list = list()
                for i in _case:
                    a = i.replace("\033[1;37m", '')
                    a = a.replace("\033[1;32m", '')
                    a = a.replace("\033[1;30m", '')
                    a = a.replace("\033[0m", '')
                    a = a.replace("\033[A", '')
                    a = a.replace("\033[0;36m", '')
                    if not a.startswith("Feature"):
                        if a in temp_list:
                            pass
                        else:
                            temp_list.append(a)
                            if world.fail_flag is True or world.skip_flag is True:

                                # 收集错误案例详情，支持-lf指令执行上次失败的案例
                                if "\033[0;31m" not in a and "\033[1;31m" not in a:
                                    if "#" in a:
                                        c.write(a.split('#')[0].rstrip() + '\n')
                                    elif "|" in a:
                                        c.write(a)

                                # 收集错误信息，保存在结果文件
                                a = a.replace("\033[0;31m", 'error!')
                                a = a.replace("\033[1;31m", 'error!')
                                a = a.replace("\033[1;41;33m", '')
                                f.write(f'{a}\n')
                            else:
                                # 创建临时文件，此处存放执行通过的案例，如果存在执行失败的案例此处临时文件会与错误案例文件合并，
                                # 生成新案例文件,支持-ff指令优先执行上次执行失败的案例
                                if "#" in a:
                                    t.write(a.split('#')[0].rstrip() + '\n')
                                elif "|" in a:
                                    t.write(a)
                if world.fail_flag is True or world.skip_flag is True:
                    world.case_log_name = int(time.time())
                    f.write(f'日志详情见：{world.case_log_name}')
                
    world.fail_flag = False
    world.skip_flag = False
    world.case = []


def case_data():
    temp_list = list()
    _case = world.case[:]
    for i in _case:
        a = i.replace("\033[1;37m", '')
        a = a.replace("\033[1;32m", '')
        a = a.replace("\033[1;30m", '')
        a = a.replace("\033[0m", '')
        a = a.replace("\033[A", '')
        a = a.replace("\033[0;36m", '')
        if not a.startswith("Feature"):
            if a in temp_list:
                ...
            else:
                temp_list.append(a)
                yield a


def build_case(scenario):
    
    if world.fail_flag is True or world.skip_flag is True:
        with open(f'{world.cache_path}/res.txt', 'a+', encoding='utf8') as f:
            with open(f'{world.cache_path}/error_case.feature', 'a+', encoding='utf8') as c:
                for a in case_data():
                    # 收集错误案例详情，支持-lf指令执行上次失败的案例
                    if "\033[0;31m" not in a and "\033[1;31m" not in a:
                        if "#" in a:
                            c.write(a.split('#')[0].rstrip() + '\n')
                        elif "|" in a:
                            c.write(a)
                    # 收集错误信息，保存在结果文件
                    a = a.replace("\033[0;31m", 'error!')
                    a = a.replace("\033[1;31m", 'error!')
                    a = a.replace("\033[1;41;33m", '')
                    f.write(f'{a}\n')                        
    else:
        with open(f'{world.cache_path}/temp_case.feature', 'a+', encoding='utf8') as t:
            for a in case_data():
                if "#" in a:
                    t.write(a.split('#')[0].rstrip() + '\n')
                elif "|" in a:
                    t.write(a)
    if world.fail_flag is True or world.skip_flag is True:
        world.case_log_name = int(time.time())
        with open(f'{world.cache_path}/res.txt', 'a+', encoding='utf8') as f:
            f.write(f'日志详情见：{world.case_log_name}')

    world.fail_flag = False
    world.skip_flag = False
    world.case = []



class CallbackDict(dict):
    def append_to(self, where, when, function):
        if not any(_function_matches(o, function) for o in self[where][when]):
            self[where][when].append(function)
            if where == 'scenario' and when == 'after_each':
                self[where][when].append(build_case)


    def clear(self):
        for name, action_dict in self.items():
            for callback_list in action_dict.values():
                callback_list[:] = []

class StepDict(dict):
    def __init__(self, *args, **kwargs):
        super(StepDict, self).__init__(*args, **kwargs)
        self._compiled = {}
        self._compiled_ignore_case = {}

    def get_regex(self, step, ignore_case=False):
        if ignore_case:
            regex = self._compiled_ignore_case.get(step, None)
            if not regex:
                regex = re.compile(step, re.I)
                self._compiled_ignore_case[step] = regex
        else:
            regex = self._compiled.get(step, None)
            if not regex:
                regex = re.compile(step)
                self._compiled[step] = regex
        return regex

    def load(self, step, func):
        self._assert_is_step(step, func)
        self[step] = func
        return func

    def load_func(self, func):
        regex = self._extract_sentence(func)
        return self.load(regex, func)

    def load_steps(self, obj):
        exclude = getattr(obj, "exclude", [])
        for attr in dir(obj):
            if self._attr_is_step(attr, obj) and attr not in exclude:
                step_method = getattr(obj, attr)
                self.load_func(step_method)
        return obj

    def _extract_sentence(self, func):
        func = getattr(func, '__func__', func)
        sentence = getattr(func, '__doc__', None)
        if sentence is None:
            if six.PY2:
                sentence = func.func_name.replace('_', ' ')
            if six.PY3:
                sentence = func.__name__.replace('_', ' ')
            sentence = sentence[0].upper() + sentence[1:]
        return sentence

    def _assert_is_step(self, step, func):
        try:
            re.compile(step)
        except re.error as e:
            raise StepLoadingError("Error when trying to compile:\n"
                                   "  regex: %r\n"
                                   "  for function: %s\n"
                                   "  error: %s" % (step, func, e))

    def _attr_is_step(self, attr, obj):
        return attr[0] != '_' and self._is_func_or_method(getattr(obj, attr))

    def _is_func_or_method(self, func):
        func_dir = dir(func)
        return callable(func) and ("func_name" in func_dir or "__func__" in func_dir)


STEP_REGISTRY = StepDict()
CALLBACK_REGISTRY = CallbackDict(
    {
        'all': {
            'before': [],
            'after': [],
        },
        'step': {
            'before_each': [],
            'after_each': [],
            'before_output': [],
            'after_output': [],
        },
        'scenario': {
            'before_each': [],
            'after_each': [],
            'outline': [],
        },
        'outline': {
            'before_each': [],
            'after_each': [],
        },
        'background': {
            'before_each': [],
            'after_each': [],
        },
        'feature': {
            'before_each': [],
            'after_each': [],
        },
        'app': {
            'before_each': [],
            'after_each': [],
        },
        'harvest': {
            'before': [],
            'after': [],
        },
        'handle_request': {
            'before': [],
            'after': [],
        },
        'runserver': {
            'before': [],
            'after': [],
        },
    },
)


def call_hook(situation, kind, *args, **kw):
    for callback in CALLBACK_REGISTRY[kind][situation]:
        try:
            callback(*args, **kw)
        except Exception as e:
            # py3: mod
            print("=" * 1000)
            if six.PY2:
                traceback.print_exc(e)
            if six.PY3:
                traceback.print_exc()
            print
            raise


def clear():
    STEP_REGISTRY.clear()
    CALLBACK_REGISTRY.clear()



import os
import sys
from setuptools import setup


with open("README.md", "r", encoding='utf8') as fh:
    long_description = fh.read()


def get_packages():
    # setuptools can't do the job :(
    packages = []
    for root, dirnames, filenames in os.walk('lettuce'):
        if '__init__.py' in filenames:
            packages.append(".".join(os.path.split(root)).strip("."))

    return packages


required_modules = ['sure', 'fuzzywuzzy', 'python-subunit']


if os.name.lower() == 'nt':
    required_modules.append('colorama')

setup(
    name='lettuce_Dtester',
    version='0.2.30',
    description='Behaviour Driven Development for python',
    author='Gabriel Falcao',
    author_email='gabriel@nacaolivre.org',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='http://lettuce.it',
    packages=get_packages(),
    install_requires=required_modules,
    entry_points={
        'console_scripts': ['lettuce = lettuce.bin:main'],
        },
    package_data={
        'lettuce': ['COPYING', '*.md'],
    },
)



# -*- coding: utf-8 -*-
# <Lettuce - Behaviour Driven Development for python>
# Copyright (C) <2010-2012>  Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERsteps.pyCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from lettuce import core
from lettuce.terrain import after
from lettuce.terrain import before
from lettuce.plugins.reporter import Reporter

class NameReporter(Reporter):
    def print_scenario_running(self, scenario):
        self.wrt('%s ... ' % scenario.name)

    def print_scenario_ran(self, scenario):
        if scenario.passed:
            self.wrt("OK")
        elif scenario.failed:
            reason = self.scenarios_and_its_fails[scenario]
            if isinstance(reason.exception, AssertionError):
                self.wrt("FAILED")
            else:
                self.wrt("ERROR")
        self.wrt("\n")

reporter = NameReporter()

before.each_scenario(reporter.print_scenario_running)
after.each_scenario(reporter.print_scenario_ran)
after.each_step(reporter.store_failed_step)
after.all(reporter.print_end)


def print_no_features_found(where):
    where = core.fs.relpath(where)
    if not where.startswith(os.sep):
        where = '.%s%s' % (os.sep, where)

    reporter.wrt('Oops!\n')
    reporter.wrt('could not find features at %s\n' % where)
